@@ -1,52 +1,50 @@
 use core::task::{Context, Poll};
-use futures::AsyncWrite;
-use crate::backend::{Encode, FormatEncode, FormatSerialize};
+use crate::backend::{Encode, FormatSerialize};
+use crate::io;
+
 
 /// Implements a [serialization future](FormatSerialize) for any
 /// [encoder](Encode).
 pub struct SerializeAll<'w, F, W, Dta, Enc>
 where
-    F: FormatEncode,
-    Enc: Encode<Format=F, Data=Dta>,
+    Dta: ?Sized,
 {
     format: &'w F,
     writer: &'w mut W,
     data: &'w Dta,
-    encode: Enc,
+    encoder: Enc,
 }
 
 impl<'w, F, W, Dta, Enc> SerializeAll<'w, F, W, Dta, Enc>
 where
-    F: FormatSerialize,
-    Enc: Encode<Format=F, Data=Dta>,
+    Dta: ?Sized,
 {
-    pub fn new(format: &'w F, writer: &'w mut W, data: &'w Dta, encode: Enc) -> Self {
+    pub fn new(format: &'w F, writer: &'w mut W, data: &'w Dta, encoder: Enc) -> Self {
         Self{
             format,
             writer,
             data,
-            encode,
+            encoder,
         }
     }
 }
 
 impl<'w, F, W, Dta, Enc> Unpin for SerializeAll<'w, F, W, Dta, Enc>
 where
-    F: FormatSerialize,
-    W: Unpin,
-    Enc: Encode<Format=F, Data=Dta>,
+    Dta: ?Sized,
 {}
 
 impl<'w, F, W, Dta, Enc> core::future::Future for SerializeAll<'w, F, W, Dta, Enc>
 where
     F: FormatSerialize,
-    W: AsyncWrite + Unpin,
+    W: io::AsyncWrite + Unpin,
+    Dta: ?Sized,
     Enc: Encode<Format=F, Data=Dta>,
 {
     type Output = Result<(), F::Error>;
 
     fn poll(mut self: core::pin::Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
         let this = &mut *self;
-        this.encode.poll_encode(this.format, &mut this.writer, this.data, cx)
+        this.encoder.poll_encode(this.format, &mut this.writer, this.data, cx).into()
     }
 }
\ No newline at end of file
